@@ -4,7 +4,6 @@ import decimal
 
 from botocore.compat import six
 from botocore.model import ShapeResolver
-from botocore.model import StructureShape
 from botocore.validate import ParamValidator
 
 BOILER_PLATE_SHAPES = {
@@ -33,7 +32,6 @@ class BaseTestValidate(unittest.TestCase):
         input_shape = s.get_shape_by_name('Input')
         validator = ParamValidator()
         errors_found = validator.validate(input_params, input_shape)
-        error_message = errors_found.generate_report()
         return errors_found
 
 
@@ -94,6 +92,240 @@ class TestValidateRequiredParams(BaseTestValidate):
             errors=['Unknown parameter'])
 
 
+class TestValidateJSONValueTrait(BaseTestValidate):
+    def test_accepts_jsonvalue_string(self):
+        self.shapes = {
+            'Input': {
+                'type': 'structure',
+                'members': {
+                    'json': {
+                        'shape': 'StrType',
+                        'jsonvalue': True,
+                        'location': 'header',
+                        'locationName': 'header-name'
+                    }
+                }
+            },
+            'StrType': {'type': 'string'}
+        }
+        errors = self.get_validation_error_message(
+            given_shapes=self.shapes,
+            input_params={
+                'json': {'data': [1, 2.3, '3'], 'unicode': u'\u2713'}
+            })
+        error_msg = errors.generate_report()
+        self.assertEqual(error_msg, '')
+
+    def test_validate_jsonvalue_string(self):
+        self.shapes = {
+            'Input': {
+                'type': 'structure',
+                'members': {
+                    'json': {
+                        'shape': 'StrType',
+                        'jsonvalue': True,
+                        'location': 'header',
+                        'locationName': 'header-name'
+                    }
+                }
+            },
+            'StrType': {'type': 'string'}
+        }
+
+        self.assert_has_validation_errors(
+            given_shapes=self.shapes,
+            input_params={
+                'json': {'date': datetime(2017, 4, 27, 0, 0)}
+            },
+            errors=[
+                ('Invalid parameter json must be json serializable: ')
+            ])
+
+
+class TestValidateDocumentType(BaseTestValidate):
+    def test_accepts_document_type_string(self):
+        self.shapes = {
+            'Input': {
+                'type': 'structure',
+                'members': {
+                    'inlineDocument': {
+                        'shape': 'DocumentType',
+                    }
+                }
+            },
+            'DocumentType': {
+                'type': 'structure',
+                'document': True
+            }
+        }
+        errors = self.get_validation_error_message(
+            given_shapes=self.shapes,
+            input_params={
+                'inlineDocument': {'data': [1, 2.3, '3',
+                                            {'foo': None}],
+                                   'unicode': u'\u2713'}
+            })
+        error_msg = errors.generate_report()
+        self.assertEqual(error_msg, '')
+
+    def test_validate_document_type_string(self):
+        self.shapes = {
+            'Input': {
+                'type': 'structure',
+                'members': {
+                    'inlineDocument': {
+                        'shape': 'DocumentType',
+                    }
+                }
+            },
+            'DocumentType': {
+                'type': 'structure',
+                'document': True
+            }
+        }
+
+        invalid_document = object()
+        self.assert_has_validation_errors(
+            given_shapes=self.shapes,
+            input_params={
+                'inlineDocument': {
+                    'number': complex(1j),
+                    'date': datetime(2017, 4, 27, 0, 0),
+                    'list': [invalid_document],
+                    'dict': {'foo': (1, 2, 3)}
+                }
+            },
+            errors=[
+                ('Invalid type for document parameter number'),
+                ('Invalid type for document parameter date'),
+                ('Invalid type for document parameter list[0]'),
+                ('Invalid type for document parameter foo'),
+            ])
+
+
+class TestValidateTaggedUnion(BaseTestValidate):
+    def test_accepts_one_member(self):
+        self.shapes = {
+            'Input': {
+                'type': 'structure',
+                'members': {
+                    'taggedUnion': {
+                        'shape': 'TaggedUnionType',
+                    }
+                }
+            },
+            'TaggedUnionType': {
+                'type': 'structure',
+                'union': True,
+                'members': {
+                    'Foo': {'shape': 'StringType'},
+                    'Bar': {'shape': 'StringType'},
+                }
+            },
+            'StringType': {'type': 'string'}
+        }
+        errors = self.get_validation_error_message(
+            given_shapes=self.shapes,
+            input_params={
+                'taggedUnion': {'Foo': "mystring"}
+            }
+        )
+        error_msg = errors.generate_report()
+        self.assertEqual(error_msg, '')
+
+    def test_validate_one_member_is_set(self):
+        self.shapes = {
+            'Input': {
+                'type': 'structure',
+                'members': {
+                    'taggedUnion': {
+                        'shape': 'TaggedUnionType',
+                    }
+                }
+            },
+            'TaggedUnionType': {
+                'type': 'structure',
+                'union': True,
+                'members': {
+                    'Foo': {'shape': 'StringType'},
+                    'Bar': {'shape': 'StringType'},
+                }
+            },
+            'StringType': {'type': 'string'}
+        }
+        errors = self.get_validation_error_message(
+            given_shapes=self.shapes,
+            input_params={
+                'taggedUnion': {'Foo': "mystring",
+                                'Bar': "mystring2"
+                                }
+            }
+        )
+        error_msg = errors.generate_report()
+        self.assertIn(
+            'Invalid number of parameters set for tagged union structure',
+            error_msg
+        )
+
+    def test_validate_known_member_is_set(self):
+        self.shapes = {
+            'Input': {
+                'type': 'structure',
+                'members': {
+                    'taggedUnion': {
+                        'shape': 'TaggedUnionType',
+                    }
+                }
+            },
+            'TaggedUnionType': {
+                'type': 'structure',
+                'union': True,
+                'members': {
+                    'Foo': {'shape': 'StringType'},
+                    'Bar': {'shape': 'StringType'},
+                }
+            },
+            'StringType': {'type': 'string'}
+        }
+        errors = self.get_validation_error_message(
+            given_shapes=self.shapes,
+            input_params={
+                'taggedUnion': {'unknown': "mystring"}
+            }
+        )
+        error_msg = errors.generate_report()
+        self.assertIn('Unknown parameter in taggedUnion', error_msg)
+
+    def test_validate_structure_is_not_empty(self):
+        self.shapes = {
+            'Input': {
+                'type': 'structure',
+                'members': {
+                    'taggedUnion': {
+                        'shape': 'TaggedUnionType',
+                    }
+                }
+            },
+            'TaggedUnionType': {
+                'type': 'structure',
+                'union': True,
+                'members': {
+                    'Foo': {'shape': 'StringType'},
+                    'Bar': {'shape': 'StringType'},
+                }
+            },
+            'StringType': {'type': 'string'}
+        }
+        errors = self.get_validation_error_message(
+            given_shapes=self.shapes,
+            input_params={
+                'taggedUnion': {}
+            }
+        )
+        error_msg = errors.generate_report()
+        self.assertIn('Must set one of the following keys', error_msg)
+
+
 class TestValidateTypes(BaseTestValidate):
     def setUp(self):
         self.shapes = {
@@ -152,7 +384,7 @@ class TestValidateTypes(BaseTestValidate):
     def test_datetime_type_accepts_datetime_obj(self):
         errors = self.get_validation_error_message(
             given_shapes=self.shapes,
-            input_params={'Timestamp': datetime.now(),})
+            input_params={'Timestamp': datetime.now()})
         error_msg = errors.generate_report()
         self.assertEqual(error_msg, '')
 
@@ -229,8 +461,8 @@ class TestValidateRanges(BaseTestValidate):
                 'Long': -10,
             },
             errors=[
-                'Invalid range for parameter Int',
-                'Invalid range for parameter Long',
+                'Invalid value for parameter Int',
+                'Invalid value for parameter Long',
             ]
         )
 
@@ -374,7 +606,7 @@ class TestValidationFloatType(BaseTestValidate):
                 'Float': 1,
             },
             errors=[
-                'Invalid range for parameter Float',
+                'Invalid value for parameter Float',
             ]
         )
 
@@ -392,7 +624,7 @@ class TestValidationFloatType(BaseTestValidate):
                 'Float': decimal.Decimal('1'),
             },
             errors=[
-                'Invalid range for parameter Float',
+                'Invalid value for parameter Float',
             ]
         )
 
